@@ -1,27 +1,27 @@
 #include "client.h"
 #include "protocol.h"
 
-#include "base/coded.h"
 #include "base/compressed.h"
 #include "base/socket.h"
 #include "base/wire_format.h"
 
 #include "columns/factory.h"
 
-#include <cityhash/city.h>
-#include <lz4/lz4.h>
-
 #include <assert.h>
 #include <atomic>
 #include <system_error>
 #include <thread>
 #include <vector>
 #include <sstream>
+#include <stdexcept>
+
+#if defined(WITH_OPENSSL)
+#include "base/sslsocket.h"
+#endif
 
 #define DBMS_NAME                                       "ClickHouse"
-#define DBMS_VERSION_MAJOR                              1
+#define DBMS_VERSION_MAJOR                              2
 #define DBMS_VERSION_MINOR                              1
-#define REVISION                                        54126
 
 #define DBMS_MIN_REVISION_WITH_TEMPORARY_TABLES         50264
 #define DBMS_MIN_REVISION_WITH_TOTAL_ROWS_IN_PROGRESS   51554
@@ -29,6 +29,13 @@
 #define DBMS_MIN_REVISION_WITH_CLIENT_INFO              54032
 #define DBMS_MIN_REVISION_WITH_SERVER_TIMEZONE          54058
 #define DBMS_MIN_REVISION_WITH_QUOTA_KEY_IN_CLIENT_INFO 54060
+//#define DBMS_MIN_REVISION_WITH_TABLES_STATUS            54226
+#define DBMS_MIN_REVISION_WITH_TIME_ZONE_PARAMETER_IN_DATETIME_DATA_TYPE 54337
+#define DBMS_MIN_REVISION_WITH_SERVER_DISPLAY_NAME      54372
+#define DBMS_MIN_REVISION_WITH_VERSION_PATCH            54401
+#define DBMS_MIN_REVISION_WITH_LOW_CARDINALITY_TYPE     54405
+
+#define REVISION  DBMS_MIN_REVISION_WITH_LOW_CARDINALITY_TYPE
 
 namespace clickhouse {
 
@@ -44,55 +51,91 @@ struct ClientInfo {
     std::string initial_address = "[::ffff:127.0.0.1]:0";
     uint64_t client_version_major = 0;
     uint64_t client_version_minor = 0;
+    uint64_t client_version_patch = 0;
     uint32_t client_revision = 0;
 };
 
-struct ServerInfo {
-    std::string name;
-    std::string timezone;
-    uint64_t    version_major;
-    uint64_t    version_minor;
-    uint64_t    revision;
-};
-
 std::ostream& operator<<(std::ostream& os, const ClientOptions& opt) {
     os << "Client(" << opt.user << '@' << opt.host << ":" << opt.port
        << " ping_before_query:" << opt.ping_before_query
        << " send_retries:" << opt.send_retries
        << " retry_timeout:" << opt.retry_timeout.count()
        << " compression_method:"
-       << (opt.compression_method == CompressionMethod::LZ4 ? "LZ4" : "None")
-       << ")";
+       << (opt.compression_method == CompressionMethod::LZ4 ? "LZ4" : "None");
+#if defined(WITH_OPENSSL)
+    if (opt.ssl_options) {
+        const auto & ssl_options = *opt.ssl_options;
+        os << " SSL ("
+           << " ssl_context: " << (ssl_options.ssl_context ? "provided by user" : "created internally")
+           << " use_default_ca_locations: " << ssl_options.use_default_ca_locations
+           << " path_to_ca_files: " << ssl_options.path_to_ca_files.size() << " items"
+           << " path_to_ca_directory: " << ssl_options.path_to_ca_directory
+           << " min_protocol_version: " << ssl_options.min_protocol_version
+           << " max_protocol_version: " << ssl_options.max_protocol_version
+           << " context_options: " << ssl_options.context_options
+           << ")";
+    }
+#endif
+    os << ")";
     return os;
 }
 
+ClientOptions& ClientOptions::SetSSLOptions(ClientOptions::SSLOptions options)
+{
+#ifdef WITH_OPENSSL
+    ssl_options = options;
+    return *this;
+#else
+    (void)options;
+    throw std::runtime_error("Library was built with no SSL support");
+#endif
+}
+
+namespace {
+
+std::unique_ptr<SocketFactory> GetSocketFactory(const ClientOptions& opts) {
+    (void)opts;
+#if defined(WITH_OPENSSL)
+    if (opts.ssl_options)
+        return std::make_unique<SSLSocketFactory>(opts);
+    else
+#endif
+        return std::make_unique<NonSecureSocketFactory>();
+}
+
+}
+
 class Client::Impl {
 public:
      Impl(const ClientOptions& opts);
+     Impl(const ClientOptions& opts,
+          std::unique_ptr<SocketFactory> socket_factory);
     ~Impl();
 
     void ExecuteQuery(Query query);
 
     void SendCancel();
 
-    void Insert(const std::string& table_name, const Block& block);
+    void Insert(const std::string& table_name, const std::string& query_id, const Block& block);
 
     void Ping();
 
     void ResetConnection();
 
+    const ServerInfo& GetServerInfo() const;
+
 private:
     bool Handshake();
 
     bool ReceivePacket(uint64_t* server_packet = nullptr);
 
-    void SendQuery(const std::string& query);
+    void SendQuery(const std::string& query, const std::string& query_id);
 
     void SendData(const Block& block);
 
     bool SendHello();
 
-    bool ReadBlock(Block* block, CodedInputStream* input);
+    bool ReadBlock(InputStream& input, Block* block);
 
     bool ReceiveHello();
 
@@ -102,7 +145,9 @@ private:
     /// Reads exception packet form input stream.
     bool ReceiveException(bool rethrow = false);
 
-    void WriteBlock(const Block& block, CodedOutputStream* output);
+    void WriteBlock(const Block& block, OutputStream& output);
+
+    void InitializeStreams(std::unique_ptr<SocketBase>&& socket);
 
 private:
     /// In case of network errors tries to reconnect to server and
@@ -136,32 +181,30 @@ private:
     QueryEvents* events_;
     int compression_ = CompressionState::Disable;
 
-    SocketHolder socket_;
+    std::unique_ptr<SocketFactory> socket_factory_;
 
-    SocketInput socket_input_;
-    BufferedInput buffered_input_;
-    CodedInputStream input_;
+    std::unique_ptr<InputStream> input_;
+    std::unique_ptr<OutputStream> output_;
+    std::unique_ptr<SocketBase> socket_;
 
-    SocketOutput socket_output_;
-    BufferedOutput buffered_output_;
-    CodedOutputStream output_;
+#if defined(WITH_OPENSSL)
+    std::unique_ptr<SSLContext> ssl_context_;
+#endif
 
     ServerInfo server_info_;
 };
 
 
 Client::Impl::Impl(const ClientOptions& opts)
+    : Impl(opts, GetSocketFactory(opts)) {}
+
+Client::Impl::Impl(const ClientOptions& opts,
+                   std::unique_ptr<SocketFactory> socket_factory)
     : options_(opts)
     , events_(nullptr)
-    , socket_(-1)
-    , socket_input_(socket_)
-    , buffered_input_(&socket_input_)
-    , input_(&buffered_input_)
-    , socket_output_(socket_)
-    , buffered_output_(&socket_output_)
-    , output_(&buffered_output_)
+    , socket_factory_(std::move(socket_factory))
 {
-    for (int i = 0; ; ) {
+    for (unsigned int i = 0; ; ) {
         try {
             ResetConnection();
             break;
@@ -170,7 +213,7 @@ Client::Impl::Impl(const ClientOptions& opts)
                 throw;
             }
 
-            std::this_thread::sleep_for(options_.retry_timeout);
+            socket_factory_->sleepFor(options_.retry_timeout);
         }
     }
 
@@ -189,37 +232,49 @@ void Client::Impl::ExecuteQuery(Query query) {
         RetryGuard([this]() { Ping(); });
     }
 
-    SendQuery(query.GetText());
+    SendQuery(query.GetText(), query.GetQueryID());
 
     while (ReceivePacket()) {
         ;
     }
 }
 
-void Client::Impl::Insert(const std::string& table_name, const Block& block) {
+std::string NameToQueryString(const std::string &input)
+{
+    std::string output;
+    output.reserve(input.size() + 2);
+    output += '`';
+
+    for (const auto & c : input) {
+        if (c == '`') {
+            //escape ` with ``
+            output.append("``");
+        } else {
+            output.push_back(c);
+        }
+    }
+
+    output += '`';
+    return output;
+}
+
+void Client::Impl::Insert(const std::string& table_name, const std::string& query_id, const Block& block) {
     if (options_.ping_before_query) {
         RetryGuard([this]() { Ping(); });
     }
 
-    std::vector<std::string> fields;
-    fields.reserve(block.GetColumnCount());
-
-    // Enumerate all fields
-    for (unsigned int i = 0; i < block.GetColumnCount(); i++) {
-        fields.push_back(block.GetColumnName(i));
-    }
-
     std::stringstream fields_section;
+        const auto num_columns = block.GetColumnCount();
 
-    for (auto elem = fields.begin(); elem != fields.end(); ++elem) {
-        if (std::distance(elem, fields.end()) == 1) {
-            fields_section << *elem;
+    for (unsigned int i = 0; i < num_columns; ++i) {
+        if (i == num_columns - 1) {
+            fields_section << NameToQueryString(block.GetColumnName(i));
         } else {
-            fields_section << *elem << ",";
+            fields_section << NameToQueryString(block.GetColumnName(i)) << ",";
         }
     }
 
-    SendQuery("INSERT INTO " + table_name + " ( " + fields_section.str() + " ) VALUES");
+    SendQuery("INSERT INTO " + table_name + " ( " + fields_section.str() + " ) VALUES", query_id);
 
     uint64_t server_packet;
     // Receive data packet.
@@ -244,14 +299,21 @@ void Client::Impl::Insert(const std::string& table_name, const Block& block) {
     SendData(Block());
 
     // Wait for EOS.
-    while (ReceivePacket()) {
+    uint64_t eos_packet{0};
+    while (ReceivePacket(&eos_packet)) {
         ;
     }
+
+    if (eos_packet != ServerCodes::EndOfStream && eos_packet != ServerCodes::Exception
+        && eos_packet != ServerCodes::Log && options_.rethrow_exceptions) {
+        throw std::runtime_error(std::string{"unexpected packet from server while receiving end of query, expected (expected Exception, EndOfStream or Log, got: "}
+                            + (eos_packet ? std::to_string(eos_packet) : "nothing") + ")");
+    }
 }
 
 void Client::Impl::Ping() {
-    WireFormat::WriteUInt64(&output_, ClientCodes::Ping);
-    output_.Flush();
+    WireFormat::WriteUInt64(*output_, ClientCodes::Ping);
+    output_->Flush();
 
     uint64_t server_packet;
     const bool ret = ReceivePacket(&server_packet);
@@ -262,29 +324,17 @@ void Client::Impl::Ping() {
 }
 
 void Client::Impl::ResetConnection() {
-    SocketHolder s(SocketConnect(NetworkAddress(options_.host, std::to_string(options_.port))));
-
-    if (s.Closed()) {
-        throw std::system_error(errno, std::system_category());
-    }
-
-    if (options_.tcp_keepalive) {
-        s.SetTcpKeepAlive(options_.tcp_keepalive_idle.count(),
-                          options_.tcp_keepalive_intvl.count(),
-                          options_.tcp_keepalive_cnt);
-    }
-
-    socket_ = std::move(s);
-    socket_input_ = SocketInput(socket_);
-    socket_output_ = SocketOutput(socket_);
-    buffered_input_.Reset();
-    buffered_output_.Reset();
+    InitializeStreams(socket_factory_->connect(options_));
 
     if (!Handshake()) {
         throw std::runtime_error("fail to connect to " + options_.host);
     }
 }
 
+const ServerInfo& Client::Impl::GetServerInfo() const {
+    return server_info_;
+}
+
 bool Client::Impl::Handshake() {
     if (!SendHello()) {
         return false;
@@ -298,7 +348,7 @@ bool Client::Impl::Handshake() {
 bool Client::Impl::ReceivePacket(uint64_t* server_packet) {
     uint64_t packet_type = 0;
 
-    if (!input_.ReadVarint64(&packet_type)) {
+    if (!WireFormat::ReadVarint64(*input_, &packet_type)) {
         return false;
     }
     if (server_packet) {
@@ -321,22 +371,22 @@ bool Client::Impl::ReceivePacket(uint64_t* server_packet) {
     case ServerCodes::ProfileInfo: {
         Profile profile;
 
-        if (!WireFormat::ReadUInt64(&input_, &profile.rows)) {
+        if (!WireFormat::ReadUInt64(*input_, &profile.rows)) {
             return false;
         }
-        if (!WireFormat::ReadUInt64(&input_, &profile.blocks)) {
+        if (!WireFormat::ReadUInt64(*input_, &profile.blocks)) {
             return false;
         }
-        if (!WireFormat::ReadUInt64(&input_, &profile.bytes)) {
+        if (!WireFormat::ReadUInt64(*input_, &profile.bytes)) {
             return false;
         }
-        if (!WireFormat::ReadFixed(&input_, &profile.applied_limit)) {
+        if (!WireFormat::ReadFixed(*input_, &profile.applied_limit)) {
             return false;
         }
-        if (!WireFormat::ReadUInt64(&input_, &profile.rows_before_limit)) {
+        if (!WireFormat::ReadUInt64(*input_, &profile.rows_before_limit)) {
             return false;
         }
-        if (!WireFormat::ReadFixed(&input_, &profile.calculated_rows_before_limit)) {
+        if (!WireFormat::ReadFixed(*input_, &profile.calculated_rows_before_limit)) {
             return false;
         }
 
@@ -350,14 +400,14 @@ bool Client::Impl::ReceivePacket(uint64_t* server_packet) {
     case ServerCodes::Progress: {
         Progress info;
 
-        if (!WireFormat::ReadUInt64(&input_, &info.rows)) {
+        if (!WireFormat::ReadUInt64(*input_, &info.rows)) {
             return false;
         }
-        if (!WireFormat::ReadUInt64(&input_, &info.bytes)) {
+        if (!WireFormat::ReadUInt64(*input_, &info.bytes)) {
             return false;
         }
         if (REVISION >= DBMS_MIN_REVISION_WITH_TOTAL_ROWS_IN_PROGRESS) {
-            if (!WireFormat::ReadUInt64(&input_, &info.total_rows)) {
+            if (!WireFormat::ReadUInt64(*input_, &info.total_rows)) {
                 return false;
             }
         }
@@ -388,7 +438,7 @@ bool Client::Impl::ReceivePacket(uint64_t* server_packet) {
     return false;
 }
 
-bool Client::Impl::ReadBlock(Block* block, CodedInputStream* input) {
+bool Client::Impl::ReadBlock(InputStream& input, Block* block) {
     // Additional information about block.
     if (REVISION >= DBMS_MIN_REVISION_WITH_BLOCK_INFO) {
         uint64_t num;
@@ -424,10 +474,12 @@ bool Client::Impl::ReadBlock(Block* block, CodedInputStream* input) {
         return false;
     }
 
-    for (size_t i = 0; i < num_columns; ++i) {
-        std::string name;
-        std::string type;
+    CreateColumnByTypeSettings create_column_settings;
+    create_column_settings.low_cardinality_as_wrapped_column = options_.backward_compatibility_lowcardinality_as_wrapped_column;
 
+    std::string name;
+    std::string type;
+    for (size_t i = 0; i < num_columns; ++i) {
         if (!WireFormat::ReadString(input, &name)) {
             return false;
         }
@@ -435,9 +487,9 @@ bool Client::Impl::ReadBlock(Block* block, CodedInputStream* input) {
             return false;
         }
 
-        if (ColumnRef col = CreateColumnByType(type)) {
-            if (num_rows && !col->Load(input, num_rows)) {
-                throw std::runtime_error("can't load");
+        if (ColumnRef col = CreateColumnByType(type, create_column_settings)) {
+            if (num_rows && !col->Load(&input, num_rows)) {
+                throw std::runtime_error("can't load column '" + name + "' of type " + type);
             }
 
             block->AppendColumn(name, col);
@@ -453,22 +505,18 @@ bool Client::Impl::ReceiveData() {
     Block block;
 
     if (REVISION >= DBMS_MIN_REVISION_WITH_TEMPORARY_TABLES) {
-        std::string table_name;
-
-        if (!WireFormat::ReadString(&input_, &table_name)) {
+        if (!WireFormat::SkipString(*input_)) {
             return false;
         }
     }
 
     if (compression_ == CompressionState::Enable) {
-        CompressedInput compressed(&input_);
-        CodedInputStream coded(&compressed);
-
-        if (!ReadBlock(&block, &coded)) {
+        CompressedInput compressed(input_.get());
+        if (!ReadBlock(compressed, &block)) {
             return false;
         }
     } else {
-        if (!ReadBlock(&block, &input_)) {
+        if (!ReadBlock(*input_, &block)) {
             return false;
         }
     }
@@ -487,23 +535,29 @@ bool Client::Impl::ReceiveException(bool rethrow) {
     std::unique_ptr<Exception> e(new Exception);
     Exception* current = e.get();
 
+    bool exception_received = true;
     do {
         bool has_nested = false;
 
-        if (!WireFormat::ReadFixed(&input_, &current->code)) {
-            return false;
+        if (!WireFormat::ReadFixed(*input_, &current->code)) {
+           exception_received = false;
+           break;
         }
-        if (!WireFormat::ReadString(&input_, &current->name)) {
-            return false;
+        if (!WireFormat::ReadString(*input_, &current->name)) {
+            exception_received = false;
+            break;
         }
-        if (!WireFormat::ReadString(&input_, &current->display_text)) {
-            return false;
+        if (!WireFormat::ReadString(*input_, &current->display_text)) {
+            exception_received = false;
+            break;
         }
-        if (!WireFormat::ReadString(&input_, &current->stack_trace)) {
-            return false;
+        if (!WireFormat::ReadString(*input_, &current->stack_trace)) {
+            exception_received = false;
+            break;
         }
-        if (!WireFormat::ReadFixed(&input_, &has_nested)) {
-            return false;
+        if (!WireFormat::ReadFixed(*input_, &has_nested)) {
+            exception_received = false;
+            break;
         }
 
         if (has_nested) {
@@ -522,17 +576,17 @@ bool Client::Impl::ReceiveException(bool rethrow) {
         throw ServerException(std::move(e));
     }
 
-    return true;
+    return exception_received;
 }
 
 void Client::Impl::SendCancel() {
-    WireFormat::WriteUInt64(&output_, ClientCodes::Cancel);
-    output_.Flush();
+    WireFormat::WriteUInt64(*output_, ClientCodes::Cancel);
+    output_->Flush();
 }
 
-void Client::Impl::SendQuery(const std::string& query) {
-    WireFormat::WriteUInt64(&output_, ClientCodes::Query);
-    WireFormat::WriteString(&output_, std::string());
+void Client::Impl::SendQuery(const std::string& query, const std::string& query_id) {
+    WireFormat::WriteUInt64(*output_, ClientCodes::Query);
+    WireFormat::WriteString(*output_, query_id);
 
     /// Client info.
     if (server_info_.revision >= DBMS_MIN_REVISION_WITH_CLIENT_INFO) {
@@ -545,47 +599,50 @@ void Client::Impl::SendQuery(const std::string& query) {
         info.client_revision = REVISION;
 
 
-        WireFormat::WriteFixed(&output_, info.query_kind);
-        WireFormat::WriteString(&output_, info.initial_user);
-        WireFormat::WriteString(&output_, info.initial_query_id);
-        WireFormat::WriteString(&output_, info.initial_address);
-        WireFormat::WriteFixed(&output_, info.iface_type);
+        WireFormat::WriteFixed(*output_, info.query_kind);
+        WireFormat::WriteString(*output_, info.initial_user);
+        WireFormat::WriteString(*output_, info.initial_query_id);
+        WireFormat::WriteString(*output_, info.initial_address);
+        WireFormat::WriteFixed(*output_, info.iface_type);
 
-        WireFormat::WriteString(&output_, info.os_user);
-        WireFormat::WriteString(&output_, info.client_hostname);
-        WireFormat::WriteString(&output_, info.client_name);
-        WireFormat::WriteUInt64(&output_, info.client_version_major);
-        WireFormat::WriteUInt64(&output_, info.client_version_minor);
-        WireFormat::WriteUInt64(&output_, info.client_revision);
+        WireFormat::WriteString(*output_, info.os_user);
+        WireFormat::WriteString(*output_, info.client_hostname);
+        WireFormat::WriteString(*output_, info.client_name);
+        WireFormat::WriteUInt64(*output_, info.client_version_major);
+        WireFormat::WriteUInt64(*output_, info.client_version_minor);
+        WireFormat::WriteUInt64(*output_, info.client_revision);
 
         if (server_info_.revision >= DBMS_MIN_REVISION_WITH_QUOTA_KEY_IN_CLIENT_INFO)
-            WireFormat::WriteString(&output_, info.quota_key);
+            WireFormat::WriteString(*output_, info.quota_key);
+        if (server_info_.revision >= DBMS_MIN_REVISION_WITH_VERSION_PATCH) {
+            WireFormat::WriteUInt64(*output_, info.client_version_patch);
+        }
     }
 
     /// Per query settings.
     //if (settings)
     //    settings->serialize(*out);
     //else
-    WireFormat::WriteString(&output_, std::string());
+    WireFormat::WriteString(*output_, std::string());
 
-    WireFormat::WriteUInt64(&output_, Stages::Complete);
-    WireFormat::WriteUInt64(&output_, compression_);
-    WireFormat::WriteString(&output_, query);
+    WireFormat::WriteUInt64(*output_, Stages::Complete);
+    WireFormat::WriteUInt64(*output_, compression_);
+    WireFormat::WriteString(*output_, query);
     // Send empty block as marker of
     // end of data
     SendData(Block());
 
-    output_.Flush();
+    output_->Flush();
 }
 
 
-void Client::Impl::WriteBlock(const Block& block, CodedOutputStream* output) {
+void Client::Impl::WriteBlock(const Block& block, OutputStream& output) {
     // Additional information about block.
     if (server_info_.revision >= DBMS_MIN_REVISION_WITH_BLOCK_INFO) {
         WireFormat::WriteUInt64(output, 1);
-        WireFormat::WriteFixed (output, block.Info().is_overflows);
+        WireFormat::WriteFixed<uint8_t>(output, block.Info().is_overflows);
         WireFormat::WriteUInt64(output, 2);
-        WireFormat::WriteFixed (output, block.Info().bucket_num);
+        WireFormat::WriteFixed<int32_t>(output, block.Info().bucket_num);
         WireFormat::WriteUInt64(output, 0);
     }
 
@@ -596,73 +653,52 @@ void Client::Impl::WriteBlock(const Block& block, CodedOutputStream* output) {
         WireFormat::WriteString(output, bi.Name());
         WireFormat::WriteString(output, bi.Type()->GetName());
 
-        bi.Column()->Save(output);
+        bi.Column()->Save(&output);
     }
+    output.Flush();
 }
 
 void Client::Impl::SendData(const Block& block) {
-    WireFormat::WriteUInt64(&output_, ClientCodes::Data);
+    WireFormat::WriteUInt64(*output_, ClientCodes::Data);
 
     if (server_info_.revision >= DBMS_MIN_REVISION_WITH_TEMPORARY_TABLES) {
-        WireFormat::WriteString(&output_, std::string());
+        WireFormat::WriteString(*output_, std::string());
     }
 
     if (compression_ == CompressionState::Enable) {
-        switch (options_.compression_method) {
-            case CompressionMethod::None: {
-                assert(false);
-                break;
-            }
+        assert(options_.compression_method == CompressionMethod::LZ4);
 
-            case CompressionMethod::LZ4: {
-                Buffer tmp;
-                // Serialize block's data
-                {
-                    BufferOutput out(&tmp);
-                    CodedOutputStream coded(&out);
-                    WriteBlock(block, &coded);
-                }
-                // Reserver space for data
-                Buffer buf;
-                buf.resize(9 + LZ4_compressBound(tmp.size()));
+        std::unique_ptr<OutputStream> compressed_ouput = std::make_unique<CompressedOutput>(output_.get(), options_.max_compression_chunk_size);
+        BufferedOutput buffered(std::move(compressed_ouput), options_.max_compression_chunk_size);
 
-                // Compress data
-                int size = LZ4_compress((const char*)tmp.data(), (char*)buf.data() + 9, tmp.size());
-                buf.resize(9 + size);
-
-                // Fill header
-                uint8_t* p = buf.data();
-                // Compression method
-                WriteUnaligned(p, (uint8_t)0x82); p += 1;
-                // Compressed data size with header
-                WriteUnaligned(p, (uint32_t)buf.size()); p += 4;
-                // Original data size
-                WriteUnaligned(p, (uint32_t)tmp.size());
-
-                WireFormat::WriteFixed(&output_, CityHash128(
-                                    (const char*)buf.data(), buf.size()));
-                WireFormat::WriteBytes(&output_, buf.data(), buf.size());
-                break;
-            }
-        }
+        WriteBlock(block, buffered);
     } else {
-        WriteBlock(block, &output_);
+        WriteBlock(block, *output_);
     }
 
-    output_.Flush();
+    output_->Flush();
+}
+
+void Client::Impl::InitializeStreams(std::unique_ptr<SocketBase>&& socket) {
+    std::unique_ptr<OutputStream> output = std::make_unique<BufferedOutput>(socket->makeOutputStream());
+    std::unique_ptr<InputStream> input = std::make_unique<BufferedInput>(socket->makeInputStream());
+
+    std::swap(input, input_);
+    std::swap(output, output_);
+    std::swap(socket, socket_);
 }
 
 bool Client::Impl::SendHello() {
-    WireFormat::WriteUInt64(&output_, ClientCodes::Hello);
-    WireFormat::WriteString(&output_, std::string(DBMS_NAME) + " client");
-    WireFormat::WriteUInt64(&output_, DBMS_VERSION_MAJOR);
-    WireFormat::WriteUInt64(&output_, DBMS_VERSION_MINOR);
-    WireFormat::WriteUInt64(&output_, REVISION);
-    WireFormat::WriteString(&output_, options_.default_database);
-    WireFormat::WriteString(&output_, options_.user);
-    WireFormat::WriteString(&output_, options_.password);
+    WireFormat::WriteUInt64(*output_, ClientCodes::Hello);
+    WireFormat::WriteString(*output_, std::string(DBMS_NAME) + " client");
+    WireFormat::WriteUInt64(*output_, DBMS_VERSION_MAJOR);
+    WireFormat::WriteUInt64(*output_, DBMS_VERSION_MINOR);
+    WireFormat::WriteUInt64(*output_, REVISION);
+    WireFormat::WriteString(*output_, options_.default_database);
+    WireFormat::WriteString(*output_, options_.user);
+    WireFormat::WriteString(*output_, options_.password);
 
-    output_.Flush();
+    output_->Flush();
 
     return true;
 }
@@ -670,26 +706,38 @@ bool Client::Impl::SendHello() {
 bool Client::Impl::ReceiveHello() {
     uint64_t packet_type = 0;
 
-    if (!input_.ReadVarint64(&packet_type)) {
+    if (!WireFormat::ReadVarint64(*input_, &packet_type)) {
         return false;
     }
 
     if (packet_type == ServerCodes::Hello) {
-        if (!WireFormat::ReadString(&input_, &server_info_.name)) {
+        if (!WireFormat::ReadString(*input_, &server_info_.name)) {
             return false;
         }
-        if (!WireFormat::ReadUInt64(&input_, &server_info_.version_major)) {
+        if (!WireFormat::ReadUInt64(*input_, &server_info_.version_major)) {
             return false;
         }
-        if (!WireFormat::ReadUInt64(&input_, &server_info_.version_minor)) {
+        if (!WireFormat::ReadUInt64(*input_, &server_info_.version_minor)) {
             return false;
         }
-        if (!WireFormat::ReadUInt64(&input_, &server_info_.revision)) {
+        if (!WireFormat::ReadUInt64(*input_, &server_info_.revision)) {
             return false;
         }
 
         if (server_info_.revision >= DBMS_MIN_REVISION_WITH_SERVER_TIMEZONE) {
-            if (!WireFormat::ReadString(&input_, &server_info_.timezone)) {
+            if (!WireFormat::ReadString(*input_, &server_info_.timezone)) {
+                return false;
+            }
+        }
+
+        if (server_info_.revision >= DBMS_MIN_REVISION_WITH_SERVER_DISPLAY_NAME) {
+            if (!WireFormat::ReadString(*input_, &server_info_.display_name)) {
+                return false;
+            }
+        }
+
+        if (server_info_.revision >= DBMS_MIN_REVISION_WITH_VERSION_PATCH) {
+            if (!WireFormat::ReadUInt64(*input_, &server_info_.version_patch)) {
                 return false;
             }
         }
@@ -704,7 +752,7 @@ bool Client::Impl::ReceiveHello() {
 }
 
 void Client::Impl::RetryGuard(std::function<void()> func) {
-    for (int i = 0; i <= options_.send_retries; ++i) {
+    for (unsigned int i = 0; ; ++i) {
         try {
             func();
             return;
@@ -712,13 +760,13 @@ void Client::Impl::RetryGuard(std::function<void()> func) {
             bool ok = true;
 
             try {
-                std::this_thread::sleep_for(options_.retry_timeout);
+                socket_factory_->sleepFor(options_.retry_timeout);
                 ResetConnection();
             } catch (...) {
                 ok = false;
             }
 
-            if (!ok) {
+            if (!ok && i == options_.send_retries) {
                 throw;
             }
         }
@@ -731,6 +779,13 @@ Client::Client(const ClientOptions& opts)
 {
 }
 
+Client::Client(const ClientOptions& opts,
+               std::unique_ptr<SocketFactory> socket_factory)
+    : options_(opts)
+    , impl_(new Impl(opts, std::move(socket_factory)))
+{
+}
+
 Client::~Client()
 { }
 
@@ -742,16 +797,28 @@ void Client::Select(const std::string& query, SelectCallback cb) {
     Execute(Query(query).OnData(cb));
 }
 
+void Client::Select(const std::string& query, const std::string& query_id, SelectCallback cb) {
+    Execute(Query(query, query_id).OnData(cb));
+}
+
 void Client::SelectCancelable(const std::string& query, SelectCancelableCallback cb) {
     Execute(Query(query).OnDataCancelable(cb));
 }
 
+void Client::SelectCancelable(const std::string& query, const std::string& query_id, SelectCancelableCallback cb) {
+    Execute(Query(query, query_id).OnDataCancelable(cb));
+}
+
 void Client::Select(const Query& query) {
     Execute(query);
 }
 
 void Client::Insert(const std::string& table_name, const Block& block) {
-    impl_->Insert(table_name, block);
+    impl_->Insert(table_name, Query::default_query_id, block);
+}
+
+void Client::Insert(const std::string& table_name, const std::string& query_id, const Block& block) {
+    impl_->Insert(table_name, query_id, block);
 }
 
 void Client::Ping() {
@@ -762,4 +829,8 @@ void Client::ResetConnection() {
     impl_->ResetConnection();
 }
 
+const ServerInfo& Client::GetServerInfo() const {
+    return impl_->GetServerInfo();
+}
+
 }
