@@ -4,6 +4,12 @@
 pushd $(dirname $0) &>/dev/null
 trap 'popd &>/dev/null' EXIT
 
+echo "checking for './deps/libssl/openssl/crypto/bio/bio_local.h'"
+if [[ ! -e './openssl/crypto/bio/bio_local.h' ]]; then
+	echo "extracting 'openssl-*.tar.gz'"
+	tar -zxf openssl-*.tar.gz
+fi
+
 echo "extracting 'struct bio_st' from './deps/libssl/openssl/crypto/bio/bio_local.h'"
 DEPBIOST=$(cd ../../; cat ./deps/libssl/openssl/crypto/bio/bio_local.h | sed -n '/^struct bio_st {/,/}/p')
 
