@@ -51,6 +51,7 @@ class Query_Info {
 	MYSQL_STMT *mysql_stmt;
 	stmt_execute_metadata_t *stmt_meta;
 	uint64_t stmt_global_id;
+	uint64_t stmt_client_id;
 	MySQL_STMT_Global_info *stmt_info;
 
 	int QueryLength;
@@ -61,6 +62,7 @@ class Query_Info {
 	uint64_t affected_rows;
 	uint64_t rows_sent;
 	uint64_t waiting_since;
+	std::string show_warnings_prev_query_digest;
 
 	Query_Info();
 	~Query_Info();
@@ -360,7 +362,7 @@ class MySQL_Session
 #define KILL_CONNECTION  2
 
 class KillArgs {
-	public:
+public:
 	MySQL_Thread *mt;
 	char *username;
 	char *password;
@@ -369,8 +371,14 @@ class KillArgs {
 	unsigned long id;
 	int kill_type;
 	unsigned int hid;
-	KillArgs(char *u, char *p, char *h, unsigned int P, unsigned int _hdi, unsigned long i, int kt, MySQL_Thread *_mt);
+
+	KillArgs(char* u, char* p, char* h, unsigned int P, unsigned int _hdi, unsigned long i, int kt, MySQL_Thread* _mt);
+	KillArgs(char *u, char *p, char *h, unsigned int P, unsigned int _hdi, unsigned long i, int kt, MySQL_Thread *_mt, char *ip);
 	~KillArgs();
+	const char* get_host_address() const;
+
+private:
+	char* ip_addr;
 };
 
 void * kill_query_thread(void *arg);
