@@ -22,6 +22,7 @@ void ma_free_root(MA_MEM_ROOT *root, myf MyFLAGS);
 void *ma_alloc_root(MA_MEM_ROOT *mem_root, size_t Size);
 #define MAX(a,b) (((a) > (b)) ? (a) : (b))
 
+
 void * ma_alloc_root(MA_MEM_ROOT *mem_root, size_t Size)
 {
   size_t get_size;
@@ -449,6 +450,7 @@ MySQL_Connection::MySQL_Connection() {
 	statuses.myconnpoll_get = 0;
 	statuses.myconnpoll_put = 0;
 	memset(gtid_uuid,0,sizeof(gtid_uuid));
+	memset(&connected_host_details, 0, sizeof(connected_host_details));
 };
 
 MySQL_Connection::~MySQL_Connection() {
@@ -508,6 +510,11 @@ MySQL_Connection::~MySQL_Connection() {
 		}
 	}
 
+	if (connected_host_details.hostname)
+		free(connected_host_details.hostname);
+
+	if (connected_host_details.ip)
+		free(connected_host_details.ip);
 };
 
 bool MySQL_Connection::set_autocommit(bool _ac) {
@@ -795,6 +802,15 @@ void MySQL_Connection::connect_start() {
 						mysql->options.client_flag |= CLIENT_DEPRECATE_EOF;
 					}
 				}
+				// In case of 'fast_forward', we only enable compression if both, client and backend matches. Otherwise,
+				// we honor the behavior of a regular connection of when a connection doesn't agree on using compression
+				// during handshake, and we fallback to an uncompressed connection.
+				client_flags &= ~(CLIENT_COMPRESS); // we disable it by default
+				if (c->options.client_flag & CLIENT_COMPRESS) {
+					if (c->options.server_capabilities & CLIENT_COMPRESS) {
+						client_flags |= CLIENT_COMPRESS;
+					}
+				}
 			}
 		}
 	}
@@ -808,7 +824,38 @@ void MySQL_Connection::connect_start() {
 		}
 	}
 	if (parent->port) {
-		async_exit_status=mysql_real_connect_start(&ret_mysql, mysql, parent->address, userinfo->username, auth_password, userinfo->schemaname, parent->port, NULL, client_flags);
+
+		char* host_ip = NULL;
+		const std::string& res_ip = MySQL_Monitor::dns_lookup(parent->address, false);
+
+		if (!res_ip.empty()) {
+			if (connected_host_details.hostname) {
+				if (strcmp(connected_host_details.hostname, parent->address) != 0) {
+					free(connected_host_details.hostname);
+					connected_host_details.hostname = strdup(parent->address);
+				}
+			}
+			else {
+				connected_host_details.hostname = strdup(parent->address);
+			}
+
+			if (connected_host_details.ip) {
+				if (strcmp(connected_host_details.ip, res_ip.c_str()) != 0) {
+					free(connected_host_details.ip);
+					connected_host_details.ip = strdup(res_ip.c_str());
+				}
+			}
+			else {
+				connected_host_details.ip = strdup(res_ip.c_str());
+			}
+
+			host_ip = connected_host_details.ip;
+		}
+		else {
+			host_ip = parent->address;
+		}
+
+		async_exit_status=mysql_real_connect_start(&ret_mysql, mysql, host_ip, userinfo->username, auth_password, userinfo->schemaname, parent->port, NULL, client_flags);
 	} else {
 		async_exit_status=mysql_real_connect_start(&ret_mysql, mysql, "localhost", userinfo->username, auth_password, userinfo->schemaname, parent->port, parent->address, client_flags);
 	}
@@ -1067,6 +1114,33 @@ handler_again:
 			}
     	break;
 		case ASYNC_CONNECT_SUCCESSFUL:
+			if (mysql && ret_mysql) {
+				// we handle encryption for backend
+				//
+				// we have a similar code in MySQL_Data_Stream::attach_connection()
+				// see there for further details
+				if (mysql->options.use_ssl == 1)
+					if (myds)
+						if (myds->sess != NULL)
+							if (myds->sess->session_fast_forward == true) {
+								assert(myds->ssl==NULL);
+								if (myds->ssl == NULL) {
+									// check the definition of P_MARIADB_TLS
+									P_MARIADB_TLS * matls = (P_MARIADB_TLS *)mysql->net.pvio->ctls;
+									if (matls != NULL) {
+										myds->encrypted = true;
+										myds->ssl = (SSL *)matls->ssl;
+										myds->rbio_ssl = BIO_new(BIO_s_mem());
+										myds->wbio_ssl = BIO_new(BIO_s_mem());
+										SSL_set_bio(myds->ssl, myds->rbio_ssl, myds->wbio_ssl);
+									} else {
+										// if mysql->options.use_ssl == 1 but matls == NULL
+										// it means that ProxySQL tried to use SSL to connect to the backend
+										// but the backend didn't support SSL
+									}
+								}
+							}
+			}
 			__sync_fetch_and_add(&MyHGM->status.server_connections_connected,1);
 			__sync_fetch_and_add(&parent->connect_OK,1);
 			options.client_flag = mysql->client_flag;
@@ -1091,6 +1165,7 @@ handler_again:
 				//vio_blocking(mysql->net.vio, FALSE, 0);
 				//fcntl(mysql->net.vio->sd, F_SETFL, O_RDWR|O_NONBLOCK);
 			//}
+			MySQL_Monitor::dns_cache_update_socket(mysql->host, mysql->net.fd);
 			break;
 		case ASYNC_CONNECT_FAILED:
 			MyHGM->p_update_mysql_error_counter(p_mysql_error_type::mysql, parent->myhgc->hid, parent->address, parent->port, mysql_errno(mysql));
@@ -1479,7 +1554,8 @@ handler_again:
 			break;
 		case ASYNC_USE_RESULT_CONT:
 			{
-				if (myds->sess && myds->sess->client_myds && myds->sess->mirror==false) {
+				if (myds->sess && myds->sess->client_myds && myds->sess->mirror==false &&
+					myds->sess->status != SHOW_WARNINGS) { // see issue#4072
 					unsigned int buffered_data=0;
 					buffered_data = myds->sess->client_myds->PSarrayOUT->len * RESULTSET_BUFLEN;
 					buffered_data += myds->sess->client_myds->resultset->len * RESULTSET_BUFLEN;
@@ -1501,7 +1577,34 @@ handler_again:
 				async_fetch_row_start=false;
 				if (mysql_row) {
 					if (myds && myds->sess && myds->sess->status == SHOW_WARNINGS) {
-						proxy_warning("MySQL Warning. Level: [%s], Code: [%s], Message: [%s]\n", mysql_row[0], mysql_row[1], mysql_row[2]);
+						if (mysql_thread___verbose_query_error) {
+							MySQL_Data_Stream* client_myds = myds->sess->client_myds;
+							const char* username = "";
+							const char* schema = "";
+							const char* client_addr = "";
+							const char* digest_text = myds->sess->CurrentQuery.show_warnings_prev_query_digest.c_str();
+
+							if (client_myds) {
+								client_addr = client_myds->addr.addr ? client_myds->addr.addr : (char *)"unknown";
+
+								if (client_myds->myconn && client_myds->myconn->userinfo) {
+									username = client_myds->myconn->userinfo->username;
+									schema = client_myds->myconn->userinfo->schemaname;
+								}
+							}
+
+							proxy_warning(
+								"Warning during query on (%d,%s,%d,%lu). User '%s@%s', schema '%s', digest_text '%s', level '%s', code '%s', message '%s'\n",
+								parent->myhgc->hid, parent->address, parent->port, get_mysql_thread_id(), username, client_addr,
+								schema, digest_text, mysql_row[0], mysql_row[1], mysql_row[2]
+							);
+						} else {
+							proxy_warning(
+								"Warning during query on (%d,%s,%d,%lu). Level '%s', code '%s', message '%s'\n",
+								parent->myhgc->hid, parent->address, parent->port, get_mysql_thread_id(), mysql_row[0], mysql_row[1],
+								mysql_row[2]
+							);
+						}
 					}
 					unsigned int br=MyRS->add_row(mysql_row);
 					__sync_fetch_and_add(&parent->bytes_recv,br);
@@ -2344,16 +2447,22 @@ bool MySQL_Connection::IsKeepMultiplexEnabledVariables(char *query_digest_text)
 	} else {
 		return false;
 	}
-	//filter @@session. and @@
+	//filter @@session., @@local. and @@
 	char *match=NULL;
 	char* last_pos=NULL;
 	const int at_session_offset = strlen("@@session.");
+	const int at_local_offset = strlen("@@local."); // Alias of session
 	const int double_at_offset = strlen("@@");
 	while (query_digest_text_filter_select && (match = strcasestr(query_digest_text_filter_select,"@@session."))) {
 		memmove(match, match + at_session_offset, strlen(match) - at_session_offset);
 		last_pos = match + strlen(match) - at_session_offset;
 		*last_pos = '\0';
 	}
+	while (query_digest_text_filter_select && (match = strcasestr(query_digest_text_filter_select, "@@local."))) {
+		memmove(match, match + at_local_offset, strlen(match) - at_local_offset);
+		last_pos = match + strlen(match) - at_local_offset;
+		*last_pos = '\0';
+	}
 	while (query_digest_text_filter_select && (match = strcasestr(query_digest_text_filter_select,"@@"))) {
 		memmove(match, match + double_at_offset, strlen(match) - double_at_offset);
 		last_pos = match + strlen(match) - double_at_offset;
