@@ -68,7 +68,8 @@ const uint32_t REPORT_INTV_SEC = 5;
 #ifdef TEST_WITHASAN
 const double MAX_ALLOWED_CPU_USAGE = 5.00;
 #else
-const double MAX_ALLOWED_CPU_USAGE = 0.15;
+//const double MAX_ALLOWED_CPU_USAGE = 0.15;
+const double MAX_ALLOWED_CPU_USAGE = 0.3; // doubled it because of extra load due to cluster
 #endif
 
 int main(int argc, char** argv) {
