@@ -52,7 +52,6 @@ int uniquequeries=0;
 int histograms=-1;
 
 bool is_mariadb = false;
-bool is_cluster = false;
 unsigned int g_connect_OK=0;
 unsigned int g_connect_ERR=0;
 unsigned int g_select_OK=0;
@@ -70,7 +69,7 @@ std::mutex mtx_;
 
 std::vector<std::string> forgotten_vars {};
 
-#include "set_testing.h"
+#include "set_testing-240.h"
 
 class var_counter {
 	public:
@@ -214,11 +213,6 @@ void * my_conn_thread(void *arg) {
 					vars[el.key()] = el.value();
 				}
 			}
-			else if (el.key() == "wsrep_sync_wait") {
-				if (is_cluster) {
-					vars[el.key()] = el.value();
-				}
-			}
 			else if (el.key() == "transaction_read_only") {
 				if (is_mariadb) {
 					vars["tx_read_only"] = el.value();
@@ -443,7 +437,7 @@ int main(int argc, char *argv[]) {
 
 	std::string fileName2(std::string(cl.workdir) + "/set_testing-240.csv");
 
-	if (detect_version(cl, is_mariadb, is_cluster) != 0) {
+	if (detect_version(cl, is_mariadb) != 0) {
 		diag("Cannot detect MySQL version");
 		return exit_status();
 	}
