@@ -187,25 +187,23 @@ void queryVariables(MYSQL *mysql, json& j, std::string& paddress) {
 	// FIXME:
 	// unify the use of wsrep_sync_wait no matter if Galera is used or not
 	std::stringstream query;
+
 	if (is_mariadb) {
 		query << "SELECT /* mysql " << mysql << " " << paddress << " */ lower(variable_name), variable_value FROM information_schema.session_variables WHERE variable_name IN "
 			" ("
-			" 'sql_safe_updates', 'max_join_size', 'net_write_timeout', 'sql_select_limit', "
-			" 'sql_select_limit', 'character_set_results', 'tx_isolation', 'tx_read_only'";
+			"'tx_isolation', 'tx_read_only', 'max_statement_time'";
 	}
+	else {
+		query << "SELECT /* mysql " << mysql << " " << paddress << " */ * FROM performance_schema.session_variables WHERE variable_name IN "
+			" ("
+			"'session_track_gtids', 'transaction_isolation', 'transaction_read_only', 'max_execution_time'";
+	}
+
 	if (is_cluster) {
-		query << "SELECT /* mysql " << mysql << " " << paddress << " */ * FROM performance_schema.session_variables WHERE variable_name IN "
-			" ("
-			" 'sql_safe_updates', 'session_track_gtids', 'max_join_size', 'net_write_timeout', 'sql_select_limit', "
-			" 'sql_select_limit', 'character_set_results', 'transaction_isolation', 'transaction_read_only', "
-			" 'wsrep_sync_wait'";
-	}
-	if (!is_mariadb && !is_cluster) {
-		query << "SELECT /* mysql " << mysql << " " << paddress << " */ * FROM performance_schema.session_variables WHERE variable_name IN "
-			" ("
-			" 'sql_safe_updates', 'session_track_gtids', 'max_join_size', 'net_write_timeout', 'sql_select_limit', "
-			" 'sql_select_limit', 'character_set_results', 'transaction_isolation', 'transaction_read_only'";
+		query << ", 'wsrep_sync_wait'";
 	}
+	
+	query << ", 'sql_safe_updates', 'max_join_size', 'net_write_timeout', 'sql_select_limit', 'character_set_results'";
 	query << ", 'hostname', 'sql_log_bin', 'sql_mode', 'init_connect', 'time_zone', 'sql_auto_is_null'";
 	query << ", 'sql_auto_is_null', 'collation_connection', 'character_set_connection', 'character_set_client', 'character_set_database', 'group_concat_max_len'";
 	query << ", 'foreign_key_checks', 'unique_checks'";
@@ -214,7 +212,7 @@ void queryVariables(MYSQL *mysql, json& j, std::string& paddress) {
 	query << ", 'innodb_lock_wait_timeout', 'innodb_strict_mode', 'innodb_table_locks'";
 	query << ", 'join_buffer_size', 'lock_wait_timeout'";
 	query << ", 'sort_buffer_size', 'optimizer_switch', 'optimizer_search_depth', 'optimizer_prune_level'";
-	query << ", 'max_execution_time', 'long_query_time', 'tmp_table_size', 'max_heap_table_size'";
+	query << ", 'long_query_time', 'tmp_table_size', 'max_heap_table_size'";
 	query << ", 'lc_messages', 'lc_time_names', 'timestamp', 'max_sort_length', 'sql_big_selects'";
 	// the following variables are likely to not exist on all systems
 	for (std::vector<std::string>::const_iterator it = possible_unknown_variables.begin() ; it != possible_unknown_variables.end() ; it++) {
